@@ -34,34 +34,73 @@ SEXP do_nothing(SEXP a, SEXP b) {
     return a;
 }
 
-SEXP do_microtiming(SEXP s_times, SEXP s_expr, SEXP s_rho) {
-    nanotime_t start, end, overhead;
-    int i, n_under_overhead = 0;
-    SEXP s_ret;
-    double *ret;
-    volatile SEXP s_tmp;
-
-    UNPACK_INT(s_times, times);
-    if(!isEnvironment(s_rho)) 
-        error("'s_rho' should be an environment");
-
-    PROTECT(s_ret = allocVector(REALSXP, times));
-    ret = REAL(s_ret);
-
+nanotime_t estimate_overhead(SEXP s_rho, int rounds) {
+    int i;
+    SEXP s_expr;
+    volatile SEXP s_tmp;    
     /* Estimate minimal overhead and warm up the machine ... */
-    overhead = 1 << 31;
-    for (i = 0; i < 1 << 18; ++i) {
+    nanotime_t start, end, overhead = 1 << 31;
+    for (i = 0; i < rounds; ++i) {
         start = get_nanotime();
-        s_tmp = do_nothing(s_expr, s_rho);
+        // s_tmp = do_nothing(s_expr, s_rho);
         end = get_nanotime();
 
         const nanotime_t diff = end - start;
         if (diff > 0 && diff < overhead)
             overhead = diff;
     }
+    return overhead;
+}
+
+SEXP do_microtiming_precision(SEXP s_rho, SEXP s_times, SEXP s_warmup) {
+    UNPACK_INT(s_warmup, warmup); 
+    UNPACK_INT(s_times, times);
+    int j, i, n=0;
+    int tmp;
+    nanotime_t overhead = estimate_overhead(s_rho, warmup);
+    nanotime_t start, end, precision = 0;
+    SEXP s_ret;
+
+    PROTECT(s_ret = allocVector(REALSXP, times));
+    while (n < times) {
+        start = get_nanotime();
+        end = get_nanotime();
+        if (end - start > 0) {
+            REAL(s_ret)[n] = end - start - overhead;
+            n++;
+        }
+    }    
+    UNPROTECT(1); /* s_ret */
+    return s_ret;
+}
+
+SEXP do_microtiming(SEXP s_exprs, SEXP s_rho, SEXP s_warmup) {
+    nanotime_t start, end, overhead;
+    int i, n_under_overhead = 0;
+    R_len_t n_exprs = 0;
+    volatile SEXP s_tmp;    
+    SEXP s_ret, s_expr;
+    double *ret;
+   
+    UNPACK_INT(s_warmup, warmup);
+
+    /* Expressions */
+    n_exprs = LENGTH(s_exprs);
+
+    /* Environment in which to evaluate */
+    if(!isEnvironment(s_rho)) 
+        error("'s_rho' should be an environment");
+
+    /* Return value: */
+    PROTECT(s_ret = allocVector(REALSXP, n_exprs));
+    ret = REAL(s_ret);
+    
+    /* Estimate minimal overhead and warm up the machine ... */
+    overhead = estimate_overhead(s_rho, warmup);
 
     /* Actual timing... */
-    for (i = 0; i < times; ++i) {
+    for (i = 0; i < n_exprs; ++i) {
+        s_expr = VECTOR_ELT(s_exprs, i);
         start = get_nanotime();
         s_tmp = eval(s_expr, s_rho);
         end = get_nanotime();
@@ -84,12 +123,13 @@ SEXP do_microtiming(SEXP s_times, SEXP s_expr, SEXP s_rho) {
      */
     if (n_under_overhead > 0) {
         if (n_under_overhead == 1) {
-            warning("Estimated overhead was greater than measured evaluation time for 1 run.");
+            warning("Estimated overhead was greater than measured evaluation time inr 1 run.");
         } else {
-            warning("Estimated overhead was greater than measured evaluation time for %i runs.", n_under_overhead);
+            warning("Estimated overhead was greater than measured evaluation time in %i runs.", 
+                    n_under_overhead);
         }
     }
-
+    
     UNPROTECT(1); /* s_ret */
     return s_ret;
 }
